﻿Imports System.IO

Public Class Form1

    ' Restore previous settings.
    Private Sub Form1_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        txtStartDirectory.Text = My.Settings.StartDirectory
        cboPattern.Text = My.Settings.Pattern
    End Sub

    ' Save current settings.
    Private Sub Form1_FormClosing(ByVal sender As Object, ByVal e As System.Windows.Forms.FormClosingEventArgs) Handles Me.FormClosing
        My.Settings.StartDirectory = txtStartDirectory.Text
        My.Settings.Pattern = cboPattern.Text
        My.Settings.Save()
    End Sub

    ' Find files matching the pattern that contain the target string.
    ' If the button is the Find and Replace button, make replacements.
    Private Sub btnFind_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnFind.Click
        SearchForFiles(lstFiles, txtStartDirectory.Text, _
            cboPattern.Text, txtFind.Text, Nothing)
    End Sub
    Private Sub btnFindAndReplace_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnFindAndReplace.Click
        SearchForFiles(lstFiles, txtStartDirectory.Text, _
            cboPattern.Text, txtFind.Text, txtReplaceWith.Text)
    End Sub

    ' Let the user browse for a start directory.
    Private Sub btnPickStartDirectory_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnPickStartDirectory.Click
        fbdStartDirectory.SelectedPath = txtStartDirectory.Text
        If (fbdStartDirectory.ShowDialog() = DialogResult.OK) Then
            txtStartDirectory.Text = fbdStartDirectory.SelectedPath
        End If
    End Sub

    ' If it's Ctrl+A, select all of the control's text.
    Private Sub TextBox_KeyDown(ByVal sender As Object, ByVal e As KeyEventArgs) Handles txtFind.KeyDown, txtReplaceWith.KeyDown
        If (e.Control AndAlso e.KeyCode = Keys.A) Then
            Dim txt As TextBox = DirectCast(sender, TextBox)
            txt.SelectAll()
            e.Handled = True
        End If
    End Sub

    ' Find files matching the pattern that contain the target string
    ' and make the replacement if appropriate.
    Private Sub SearchForFiles(ByVal lst As ListBox, ByVal start_dir As String, _
     ByVal pattern As String, ByVal from_string As String, ByVal to_string As String)
        Try
            ' Clear the result ListBox.
            lstFiles.Items.Clear()

            ' Parse the patterns.
            Dim patterns() As String = ParsePatterns(pattern)

            ' If from_string is blank, don't replace.
            If (from_string.Length < 1) Then from_string = Nothing

            Dim dir_info As New DirectoryInfo(start_dir)
            SearchDirectory(lst, dir_info, patterns, from_string, to_string)

            If (from_string Is Nothing) Then
                MessageBox.Show("Found " & lst.Items.Count & " files.")
            Else
                MessageBox.Show("Made replacements in " & lst.Items.Count & " files.")
            End If
        Catch ex As Exception
            MessageBox.Show(ex.Message)
        End Try
    End Sub

    ' Find files matching the pattern that contain the target string
    ' and make the replacement if appropriate.
    Private Sub SearchDirectory(ByVal lst As ListBox, ByVal dir_info As DirectoryInfo, _
     ByVal patterns() As String, ByVal from_string As String, ByVal to_string As String)
        ' Search this directory.
        For Each pattern As String In patterns
            ' Check this pattern.
            For Each file_info As FileInfo In dir_info.GetFiles(pattern)
                ' Process this file.
                ProcessFile(lst, file_info, from_string, to_string)
            Next file_info
        Next pattern

        ' Search subdirectories.
        For Each subdir_info As DirectoryInfo In dir_info.GetDirectories()
            SearchDirectory(lst, subdir_info, patterns, from_string, to_string)
        Next subdir_info
    End Sub

    ' Replace all occurrences of from_string with to_string.
    ' Return true if there was a problem and we should stop.
    Private Sub ProcessFile(ByVal lst As ListBox, ByVal file_info As FileInfo, ByVal from_string As String, ByVal to_string As String)
        Try
            If (from_string Is Nothing) Then
                ' Add the file to the list.
                lst.Items.Add(file_info.FullName)
            Else
                ' See if the file contains from_string.
                Dim txt As String = System.IO.File.ReadAllText(file_info.FullName)
                If (txt.Contains(from_string)) Then
                    ' Add the file to the list.
                    lst.Items.Add(file_info.FullName)

                    ' See if we should make a replacement.
                    If (to_string IsNot Nothing) Then
                        System.IO.File.WriteAllText(file_info.FullName, _
                            txt.Replace(from_string, to_string))
                    End If
                End If
            End If
        Catch ex As Exception
            MessageBox.Show("Error processing file " & _
                file_info.FullName & "\n" & ex.Message)
        End Try
    End Sub

    ' Take whatever is between parentheses (if there are any),
    ' separate them, and return them in an array.
    Private Function ParsePatterns(ByVal pattern_string As String) As String()
        ' Take whatever is between the parentheses (if there are any).
        If (pattern_string.Contains("(")) Then
            pattern_string = TextBetween(pattern_string, "(", ")")
        End If

        ' Split the string at semi-colons.
        Dim result() As String = pattern_string.Split(";"c)

        ' Trim all of the patterns to remove extra space.
        For i As Integer = 0 To result.Length - 1
            result(i) = result(i).Trim()
        Next i
        Return result
    End Function

    ' Get the text between two delimiters.
    ' Let the code throw an error if a delimiter is not found.
    Private Function TextBetween(ByVal txt As String, ByVal delimiter1 As String, ByVal delimiter2 As String) As String
        ' Find the starting delimiter.
        Dim pos1 As Integer = txt.IndexOf(delimiter1)
        Dim text_start As Integer = pos1 + delimiter1.Length
        Dim pos2 As Integer = txt.IndexOf(delimiter2, text_start)
        Return txt.Substring(text_start, pos2 - text_start)
    End Function

    ' Open the double-clicked file with the system's default application.
    Private Sub lstFiles_DoubleClick(ByVal sender As Object, ByVal e As EventArgs) Handles lstFiles.DoubleClick
        For Each selected_file As String In lstFiles.SelectedItems
            System.Diagnostics.Process.Start(selected_file)
        Next selected_file
    End Sub
End Class
